// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_bank_ctl.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/10/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt_bank_ctl (
	clk						,
	rst_n					,
	srst					,

	packet_size_i			,
	frame_number_i			,
	packet_number_i			,
	total_packet_count_i	,
	time_stamp_sec_i		,
	time_stamp_ns_i			,
	fifo_wt_start_i			,
	fifo_wt_end_i			,
	mem_wt_end_i			,

	bk1_adr_i				,
	bk2_adr_i				,
	bk3_adr_i				,
	bk4_adr_i				,
	bk5_adr_i				,
	bk6_adr_i				,
	bk7_adr_i				,
	bk8_adr_i				,
	read_bank_i				,

	bk1_frm_num_o			,
	bk1_time_sec_o			,
	bk1_time_ns_o			,
	bk2_frm_num_o			,
	bk2_time_sec_o			,
	bk2_time_ns_o			,
	bk3_frm_num_o			,
	bk3_time_sec_o			,
	bk3_time_ns_o			,
	bk4_frm_num_o			,
	bk4_time_sec_o			,
	bk4_time_ns_o			,
	bk5_frm_num_o			,
	bk5_time_sec_o			,
	bk5_time_ns_o			,
	bk6_frm_num_o			,
	bk6_time_sec_o			,
	bk6_time_ns_o			,
	bk7_frm_num_o			,
	bk7_time_sec_o			,
	bk7_time_ns_o			,
	bk8_frm_num_o			,
	bk8_time_sec_o			,
	bk8_time_ns_o			,

	rx_frame_end_o			,
	rx_frame_bank_o			,
	acc_adr_o				,
	acc_adr_valid_o			
) ;

// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	PKT_SIZE_BIT	= 11 ;
	parameter	MAC_ADR_BIT		= 48 ;
	parameter	TYPE_BIT		= 16 ;
	parameter	FRM_NUM_BIT		= 16 ;
	parameter	PACKET_NUM_BIT	= 32 ;
	parameter	T_STP_SEC_BIT	= 48 ;
	parameter	T_STP_NS_BIT	= 32 ;

	parameter	AVMM_ADR_BIT	= 27 ;

	parameter	BANK_NUM		= 8 ;

// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire								clk						;
	input	wire								rst_n					;
	input	wire								srst					;

	input	wire	[   PKT_SIZE_BIT-1 : 0 ]	packet_size_i			;
	input	wire	[    FRM_NUM_BIT-1 : 0 ]	frame_number_i			;
	input	wire	[ PACKET_NUM_BIT-1 : 0 ]	packet_number_i			;
	input	wire	[ PACKET_NUM_BIT-1 : 0 ]	total_packet_count_i	;
	input	wire	[  T_STP_SEC_BIT-1 : 0 ]	time_stamp_sec_i		;
	input	wire	[   T_STP_NS_BIT-1 : 0 ]	time_stamp_ns_i			;
	input	wire								fifo_wt_start_i			;
	input	wire								fifo_wt_end_i			;
	input	wire								mem_wt_end_i			;

	input	wire	[               31 : 0 ]	bk1_adr_i				;
	input	wire	[               31 : 0 ]	bk2_adr_i				;
	input	wire	[               31 : 0 ]	bk3_adr_i				;
	input	wire	[               31 : 0 ]	bk4_adr_i				;
	input	wire	[               31 : 0 ]	bk5_adr_i				;
	input	wire	[               31 : 0 ]	bk6_adr_i				;
	input	wire	[               31 : 0 ]	bk7_adr_i				;
	input	wire	[               31 : 0 ]	bk8_adr_i				;
	input	wire	[                2 : 0 ]	read_bank_i				;

	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk1_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk1_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk1_time_ns_o			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk2_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk2_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk2_time_ns_o			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk3_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk3_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk3_time_ns_o			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk4_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk4_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk4_time_ns_o			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk5_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk5_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk5_time_ns_o			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk6_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk6_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk6_time_ns_o			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk7_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk7_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk7_time_ns_o			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	bk8_frm_num_o			;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	bk8_time_sec_o			;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	bk8_time_ns_o			;

	output	wire								rx_frame_end_o			;
	output	wire	[                2 : 0 ]	rx_frame_bank_o			;
	output	wire	[ AVMM_ADR_BIT+4-1 : 0 ]	acc_adr_o				;
	output	wire								acc_adr_valid_o			;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	reg									fifo_wt_start_dly1_ff	;
	reg									fifo_wt_start_dly2_ff	;
	reg									fifo_wt_end_dly1_ff		;
	reg									fifo_wt_end_dly2_ff		;
	reg		[                2 : 0 ]	cur_bank_ff				;
	wire	[                2 : 0 ]	next_bank				;
	reg									frame_fifo_wt_end_ff	;
	reg									rx_frame_end_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	frame_num_ff			;
	reg		[ PACKET_NUM_BIT-1 : 0 ]	packet_number_ff		;
	reg		[ PACKET_NUM_BIT-1 : 0 ]	total_packet_count_ff	;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	time_stamp_sec_ff		;
	reg		[   T_STP_NS_BIT-1 : 0 ]	time_stamp_ns_ff		;
	reg		[ PACKET_NUM_BIT-1 : 0 ]	pck_num_size_mult_ff	;
	wire	[ PACKET_NUM_BIT-1 : 0 ]	adr_offset				;
	reg		[ AVMM_ADR_BIT+4-1 : 0 ]	acc_adr_ff				;
	reg									acc_adr_valid_ff		;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk1_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk1_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk1_time_ns_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk2_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk2_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk2_time_ns_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk3_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk3_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk3_time_ns_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk4_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk4_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk4_time_ns_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk5_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk5_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk5_time_ns_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk6_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk6_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk6_time_ns_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk7_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk7_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk7_time_ns_ff			;
	reg		[    FRM_NUM_BIT-1 : 0 ]	bk8_frm_num_ff			;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	bk8_time_sec_ff			;
	reg		[   T_STP_NS_BIT-1 : 0 ]	bk8_time_ns_ff			;

// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			fifo_wt_start_dly1_ff	<= 1'b0 ;
			fifo_wt_start_dly2_ff	<= 1'b0 ;
			fifo_wt_end_dly1_ff		<= 1'b0 ;
			fifo_wt_end_dly2_ff		<= 1'b0 ;
		end
		else if ( srst ) begin
			fifo_wt_start_dly1_ff	<= 1'b0 ;
			fifo_wt_start_dly2_ff	<= 1'b0 ;
			fifo_wt_end_dly1_ff		<= 1'b0 ;
			fifo_wt_end_dly2_ff		<= 1'b0 ;
		end
		else begin
			fifo_wt_start_dly1_ff	<= fifo_wt_start_i ;
			fifo_wt_start_dly2_ff	<= fifo_wt_start_dly1_ff ;
			fifo_wt_end_dly1_ff		<= fifo_wt_end_i ;
			fifo_wt_end_dly2_ff		<= fifo_wt_end_dly1_ff ;
		end
	end

	// bank control
	assign	next_bank	= ( cur_bank_ff + 1'b1 == read_bank_i ) ?	cur_bank_ff : cur_bank_ff + 1'b1 ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			cur_bank_ff				<= 3'b000 ;
			frame_fifo_wt_end_ff	<= 1'b0 ;
			rx_frame_end_ff			<= 1'b0 ;
		end
		else if ( srst ) begin
			cur_bank_ff				<= 3'b000 ;
			frame_fifo_wt_end_ff	<= 1'b0 ;
			rx_frame_end_ff			<= 1'b0 ;
		end
		else begin
			if ( fifo_wt_end_dly2_ff && ( packet_number_ff == total_packet_count_ff ) ) begin
				cur_bank_ff				<= next_bank ;
				frame_fifo_wt_end_ff	<= 1'b1 ;
			end
			else if ( mem_wt_end_i ) begin
				frame_fifo_wt_end_ff	<= 1'b0 ;
			end
			rx_frame_end_ff			<= frame_fifo_wt_end_ff & mem_wt_end_i ;
		end
	end

	// packet parameter latch
	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			frame_num_ff			<= { FRM_NUM_BIT{1'b0} } ;
			packet_number_ff		<= { PACKET_NUM_BIT{1'b0} } ;
			total_packet_count_ff	<= { PACKET_NUM_BIT{1'b0} } ;
			time_stamp_sec_ff		<= { T_STP_SEC_BIT{1'b0} } ;
			time_stamp_ns_ff		<= { T_STP_NS_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			frame_num_ff			<= { FRM_NUM_BIT{1'b0} } ;
			packet_number_ff		<= { PACKET_NUM_BIT{1'b0} } ;
			total_packet_count_ff	<= { PACKET_NUM_BIT{1'b0} } ;
			time_stamp_sec_ff		<= { T_STP_SEC_BIT{1'b0} } ;
			time_stamp_ns_ff		<= { T_STP_NS_BIT{1'b0} } ;
		end
		else begin
			if ( fifo_wt_start_i ) begin
				frame_num_ff			<= frame_number_i ;
				packet_number_ff		<= packet_number_i - 1'b1 ;
				total_packet_count_ff	<= total_packet_count_i - 1'b1 ;
				time_stamp_sec_ff		<= time_stamp_sec_i ;
				time_stamp_ns_ff		<= time_stamp_ns_i ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			pck_num_size_mult_ff	<= { PACKET_NUM_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			pck_num_size_mult_ff	<= { PACKET_NUM_BIT{1'b0} } ;
		end
		else begin
			if ( fifo_wt_start_dly1_ff ) begin
				pck_num_size_mult_ff	<= packet_number_ff * packet_size_i ;
			end
		end
	end

	assign	adr_offset	= pck_num_size_mult_ff ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			acc_adr_ff			<= { AVMM_ADR_BIT+4{1'b0} } ;
			acc_adr_valid_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			acc_adr_ff		<= { AVMM_ADR_BIT+4{1'b0} } ;
			acc_adr_valid_ff	<= 1'b0 ;
		end
		else begin
			if ( fifo_wt_start_dly2_ff ) begin
				case ( cur_bank_ff )
					3'h0	: acc_adr_ff <= bk1_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					3'h1	: acc_adr_ff <= bk2_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					3'h2	: acc_adr_ff <= bk3_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					3'h3	: acc_adr_ff <= bk4_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					3'h4	: acc_adr_ff <= bk5_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					3'h5	: acc_adr_ff <= bk6_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					3'h6	: acc_adr_ff <= bk7_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					3'h7	: acc_adr_ff <= bk8_adr_i[AVMM_ADR_BIT+4-1:0] + adr_offset[AVMM_ADR_BIT+4-1:0] ;
					default	: acc_adr_ff <= { AVMM_ADR_BIT+4{1'bX} } ;
				endcase
			end
			acc_adr_valid_ff	<= fifo_wt_start_dly2_ff ;
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			bk1_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk1_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk1_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk2_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk2_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk2_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk3_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk3_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk3_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk4_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk4_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk4_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk5_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk5_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk5_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk6_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk6_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk6_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk7_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk7_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk7_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk8_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk8_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk8_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			bk1_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk1_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk1_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk2_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk2_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk2_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk3_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk3_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk3_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk4_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk4_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk4_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk5_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk5_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk5_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk6_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk6_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk6_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk7_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk7_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk7_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			bk8_frm_num_ff	<= { FRM_NUM_BIT{1'b0} } ;
			bk8_time_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			bk8_time_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
		end
		else begin
			if ( fifo_wt_end_dly2_ff && ( packet_number_ff == total_packet_count_ff ) ) begin
				if ( cur_bank_ff == 3'h0 ) begin
					bk1_frm_num_ff	<= frame_num_ff ;
					bk1_time_sec_ff	<= time_stamp_sec_ff ;
					bk1_time_ns_ff	<= time_stamp_ns_ff ;
				end
				if ( cur_bank_ff == 3'h1 ) begin
					bk2_frm_num_ff	<= frame_num_ff ;
					bk2_time_sec_ff	<= time_stamp_sec_ff ;
					bk2_time_ns_ff	<= time_stamp_ns_ff ;
				end
				if ( cur_bank_ff == 3'h2 ) begin
					bk3_frm_num_ff	<= frame_num_ff ;
					bk3_time_sec_ff	<= time_stamp_sec_ff ;
					bk3_time_ns_ff	<= time_stamp_ns_ff ;
				end
				if ( cur_bank_ff == 3'h3 ) begin
					bk4_frm_num_ff	<= frame_num_ff ;
					bk4_time_sec_ff	<= time_stamp_sec_ff ;
					bk4_time_ns_ff	<= time_stamp_ns_ff ;
				end
				if ( cur_bank_ff == 3'h4 ) begin
					bk5_frm_num_ff	<= frame_num_ff ;
					bk5_time_sec_ff	<= time_stamp_sec_ff ;
					bk5_time_ns_ff	<= time_stamp_ns_ff ;
				end
				if ( cur_bank_ff == 3'h5 ) begin
					bk6_frm_num_ff	<= frame_num_ff ;
					bk6_time_sec_ff	<= time_stamp_sec_ff ;
					bk6_time_ns_ff	<= time_stamp_ns_ff ;
				end
				if ( cur_bank_ff == 3'h6 ) begin
					bk7_frm_num_ff	<= frame_num_ff ;
					bk7_time_sec_ff	<= time_stamp_sec_ff ;
					bk7_time_ns_ff	<= time_stamp_ns_ff ;
				end
				if ( cur_bank_ff == 3'h7 ) begin
					bk8_frm_num_ff	<= frame_num_ff ;
					bk8_time_sec_ff	<= time_stamp_sec_ff ;
					bk8_time_ns_ff	<= time_stamp_ns_ff ;
				end
			end
		end
	end

	//---------------//
	// output singal //
	//---------------//
	assign	bk1_frm_num_o	= bk1_frm_num_ff ;
	assign	bk1_time_sec_o	= bk1_time_sec_ff ;
	assign	bk1_time_ns_o	= bk1_time_ns_ff ;
	assign	bk2_frm_num_o	= bk2_frm_num_ff ;
	assign	bk2_time_sec_o	= bk2_time_sec_ff ;
	assign	bk2_time_ns_o	= bk2_time_ns_ff ;
	assign	bk3_frm_num_o	= bk3_frm_num_ff ;
	assign	bk3_time_sec_o	= bk3_time_sec_ff ;
	assign	bk3_time_ns_o	= bk3_time_ns_ff ;
	assign	bk4_frm_num_o	= bk4_frm_num_ff ;
	assign	bk4_time_sec_o	= bk4_time_sec_ff ;
	assign	bk4_time_ns_o	= bk4_time_ns_ff ;
	assign	bk5_frm_num_o	= bk5_frm_num_ff ;
	assign	bk5_time_sec_o	= bk5_time_sec_ff ;
	assign	bk5_time_ns_o	= bk5_time_ns_ff ;
	assign	bk6_frm_num_o	= bk6_frm_num_ff ;
	assign	bk6_time_sec_o	= bk6_time_sec_ff ;
	assign	bk6_time_ns_o	= bk6_time_ns_ff ;
	assign	bk7_frm_num_o	= bk7_frm_num_ff ;
	assign	bk7_time_sec_o	= bk7_time_sec_ff ;
	assign	bk7_time_ns_o	= bk7_time_ns_ff ;
	assign	bk8_frm_num_o	= bk8_frm_num_ff ;
	assign	bk8_time_sec_o	= bk8_time_sec_ff ;
	assign	bk8_time_ns_o	= bk8_time_ns_ff ;

	assign	rx_frame_end_o	= rx_frame_end_ff ;
	assign	rx_frame_bank_o	= cur_bank_ff ;
	assign	acc_adr_o		= acc_adr_ff ;
	assign	acc_adr_valid_o	= acc_adr_valid_ff ;


endmodule

`default_nettype wire
